# This file was auto-generated by Fern from our API Definition.

import typing

from .reducto_chunking_request import ReductoChunkingRequest
from .sentence_chunking_request import SentenceChunkingRequest
from .token_overlapping_window_chunking_request import TokenOverlappingWindowChunkingRequest

DocumentIndexChunkingRequest = typing.Union[
    ReductoChunkingRequest, SentenceChunkingRequest, TokenOverlappingWindowChunkingRequest
]
