# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from .test_suite_run_external_exec_config_data import TestSuiteRunExternalExecConfigData


class TestSuiteRunExternalExecConfig(UniversalBaseModel):
    """
    Execution configuration for running a Vellum Test Suite against an external callable
    """

    data: TestSuiteRunExternalExecConfigData
    test_case_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Optionally specify a subset of test case ids to run. If not provided, all test cases within the test suite will be run by default.
    """

    type: typing.Literal["EXTERNAL"] = "EXTERNAL"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from .array_vellum_value import ArrayVellumValue  # noqa: E402, F401, I001

update_forward_refs(TestSuiteRunExternalExecConfig)
