# This file was auto-generated by Fern from our API Definition.

import typing

VellumErrorCodeEnum = typing.Union[
    typing.Literal[
        "INVALID_REQUEST",
        "INVALID_INPUTS",
        "PROVIDER_ERROR",
        "REQUEST_TIMEOUT",
        "INTERNAL_SERVER_ERROR",
        "USER_DEFINED_ERROR",
    ],
    typing.Any,
]
