# This file was auto-generated by Fern from our API Definition.

import typing

from .chat_history_input_request import ChatHistoryInputRequest
from .json_input_request import JsonInputRequest
from .string_input_request import StringInputRequest

PromptDeploymentInputRequest = typing.Union[StringInputRequest, JsonInputRequest, ChatHistoryInputRequest]
