# This file was auto-generated by Fern from our API Definition.

import typing

from .execution_array_vellum_value import ExecutionArrayVellumValue
from .execution_chat_history_vellum_value import ExecutionChatHistoryVellumValue
from .execution_error_vellum_value import ExecutionErrorVellumValue
from .execution_function_call_vellum_value import ExecutionFunctionCallVellumValue
from .execution_json_vellum_value import ExecutionJsonVellumValue
from .execution_number_vellum_value import ExecutionNumberVellumValue
from .execution_search_results_vellum_value import ExecutionSearchResultsVellumValue
from .execution_string_vellum_value import ExecutionStringVellumValue
from .execution_thinking_vellum_value import ExecutionThinkingVellumValue

ExecutionVellumValue = typing.Union[
    ExecutionStringVellumValue,
    ExecutionNumberVellumValue,
    ExecutionJsonVellumValue,
    ExecutionChatHistoryVellumValue,
    ExecutionSearchResultsVellumValue,
    ExecutionErrorVellumValue,
    ExecutionArrayVellumValue,
    ExecutionFunctionCallVellumValue,
    ExecutionThinkingVellumValue,
]
