# This file was auto-generated by Fern from our API Definition.

import typing

from .test_suite_test_case_created_bulk_result import TestSuiteTestCaseCreatedBulkResult
from .test_suite_test_case_deleted_bulk_result import TestSuiteTestCaseDeletedBulkResult
from .test_suite_test_case_rejected_bulk_result import TestSuiteTestCaseRejectedBulkResult
from .test_suite_test_case_replaced_bulk_result import TestSuiteTestCaseReplacedBulkResult

TestSuiteTestCaseBulkResult = typing.Union[
    TestSuiteTestCaseCreatedBulkResult,
    TestSuiteTestCaseReplacedBulkResult,
    TestSuiteTestCaseDeletedBulkResult,
    TestSuiteTestCaseRejectedBulkResult,
]
