# This file was auto-generated by Fern from our API Definition.

import typing

from .fulfilled_ad_hoc_execute_prompt_event import FulfilledAdHocExecutePromptEvent
from .initiated_ad_hoc_execute_prompt_event import InitiatedAdHocExecutePromptEvent
from .rejected_ad_hoc_execute_prompt_event import RejectedAdHocExecutePromptEvent
from .streaming_ad_hoc_execute_prompt_event import StreamingAdHocExecutePromptEvent

AdHocExecutePromptEvent = typing.Union[
    InitiatedAdHocExecutePromptEvent,
    StreamingAdHocExecutePromptEvent,
    FulfilledAdHocExecutePromptEvent,
    RejectedAdHocExecutePromptEvent,
]
