# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TestSuiteTestCaseRejectedBulkResult(UniversalBaseModel):
    """
    The result of a bulk operation that failed to operate on a Test Case.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    An ID that maps back to one of the initially supplied operations. Can be used to determine the result of a given operation.
    """

    type: typing.Literal["REJECTED"] = "REJECTED"
    data: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    Details about the error that occurred
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
