# This file was auto-generated by Fern from our API Definition.

import typing

from .named_scenario_input_chat_history_variable_value_request import NamedScenarioInputChatHistoryVariableValueRequest
from .named_scenario_input_json_variable_value_request import NamedScenarioInputJsonVariableValueRequest
from .named_scenario_input_string_variable_value_request import NamedScenarioInputStringVariableValueRequest

NamedScenarioInputRequest = typing.Union[
    NamedScenarioInputStringVariableValueRequest,
    NamedScenarioInputJsonVariableValueRequest,
    NamedScenarioInputChatHistoryVariableValueRequest,
]
