# This file was auto-generated by Fern from our API Definition.

import typing

WorkflowNodeResultEventState = typing.Union[
    typing.Literal["INITIATED", "STREAMING", "FULFILLED", "REJECTED"], typing.Any
]
