# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .instructor_vectorizer_config import InstructorVectorizerConfig


class HkunlpInstructorXlVectorizer(UniversalBaseModel):
    """
    Vectorizer for hkunlp/instructor-xl.
    """

    model_name: typing.Literal["hkunlp/instructor-xl"] = "hkunlp/instructor-xl"
    config: InstructorVectorizerConfig

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
