# This file was auto-generated by Fern from our API Definition.

import typing

from .reducto_chunking import ReductoChunking
from .sentence_chunking import SentenceChunking
from .token_overlapping_window_chunking import TokenOverlappingWindowChunking

DocumentIndexChunking = typing.Union[ReductoChunking, SentenceChunking, TokenOverlappingWindowChunking]
