# This file was auto-generated by Fern from our API Definition.

import typing

DocumentProcessingState = typing.Union[
    typing.Literal["QUEUED", "PROCESSING", "PROCESSED", "FAILED", "UNKNOWN"], typing.Any
]
