# This file was auto-generated by Fern from our API Definition.

import typing

LogicalOperator = typing.Union[
    typing.Literal[
        "=",
        "!=",
        "<",
        ">",
        "<=",
        ">=",
        "contains",
        "beginsWith",
        "endsWith",
        "doesNotContain",
        "doesNotBeginWith",
        "doesNotEndWith",
        "null",
        "notNull",
        "in",
        "notIn",
        "between",
        "notBetween",
        "blank",
        "notBlank",
        "coalesce",
        "accessField",
        "parseJson",
        "and",
        "or",
        "isError",
    ],
    typing.Any,
]
