# This file was auto-generated by Fern from our API Definition.

import typing

from .scenario_input_chat_history_variable_value import ScenarioInputChatHistoryVariableValue
from .scenario_input_json_variable_value import ScenarioInputJsonVariableValue
from .scenario_input_string_variable_value import ScenarioInputStringVariableValue

ScenarioInput = typing.Union[
    ScenarioInputStringVariableValue, ScenarioInputJsonVariableValue, ScenarioInputChatHistoryVariableValue
]
