# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .audio_prompt_block import AudioPromptBlock
from .document_prompt_block import DocumentPromptBlock
from .function_call_prompt_block import FunctionCallPromptBlock
from .image_prompt_block import ImagePromptBlock
from .jinja_prompt_block import JinjaPromptBlock
from .rich_text_prompt_block import RichTextPromptBlock
from .variable_prompt_block import VariablePromptBlock
from .video_prompt_block import VideoPromptBlock

if typing.TYPE_CHECKING:
    from .chat_message_prompt_block import ChatMessagePromptBlock
PromptBlock = typing.Union[
    JinjaPromptBlock,
    "ChatMessagePromptBlock",
    VariablePromptBlock,
    RichTextPromptBlock,
    FunctionCallPromptBlock,
    AudioPromptBlock,
    VideoPromptBlock,
    ImagePromptBlock,
    DocumentPromptBlock,
]
