# This file was auto-generated by Fern from our API Definition.

import typing

from .workflow_result_event_output_data_array import WorkflowResultEventOutputDataArray
from .workflow_result_event_output_data_chat_history import WorkflowResultEventOutputDataChatHistory
from .workflow_result_event_output_data_error import WorkflowResultEventOutputDataError
from .workflow_result_event_output_data_function_call import WorkflowResultEventOutputDataFunctionCall
from .workflow_result_event_output_data_json import WorkflowResultEventOutputDataJson
from .workflow_result_event_output_data_number import WorkflowResultEventOutputDataNumber
from .workflow_result_event_output_data_search_results import WorkflowResultEventOutputDataSearchResults
from .workflow_result_event_output_data_string import WorkflowResultEventOutputDataString

WorkflowResultEventOutputData = typing.Union[
    WorkflowResultEventOutputDataString,
    WorkflowResultEventOutputDataNumber,
    WorkflowResultEventOutputDataJson,
    WorkflowResultEventOutputDataChatHistory,
    WorkflowResultEventOutputDataSearchResults,
    WorkflowResultEventOutputDataArray,
    WorkflowResultEventOutputDataFunctionCall,
    WorkflowResultEventOutputDataError,
]
